@@ -43,7 +43,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
     if ( !identical(reconfig,"live") && interactive() ) {
       while ( TRUE ) {
         cat(msg,"\n")
-        response <- toupper(trimws(readline(prompt="Would you like to install now? [Y/n] ")))
+        response <- toupper(trimws(readline(prompt="Would you like to install/update it now? [Y/n] ")))
         if ( response == "N" ) return(FALSE)
         if ( response %in% c("Y","") ) return(TRUE)
       }
@@ -100,9 +100,6 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
       }
       consent <- consent || consent2
     }
-    if ( ( javaConf$javaArchitecture == 32 ) && ( osBit() == 64 ) ) {
-      warning("32-bit Java is paired with a 64-bit operating system.  To access more memory, please run 'scalaConfig(download=\"java\")'.")
-    }
     config <- c(format=4L,scalaConf,javaConf)
     if ( download.sbt ) installSoftware(installPath,"sbt",verbose=verbose)
     sbtSpecifics <- function(x,y) list(sbtCmd=x)
@@ -166,9 +163,16 @@ findExecutable <- function(mode,prettyMode,installPath,mapper,verbose=TRUE) {  #
   conf <- tryCandidate(if ( home != "" ) file.path(home,"bin",mode) else "")
   if ( ! is.null(conf) ) return(conf)
   ###
-  label <- "PATH environment variable"
-  conf <- tryCandidate(Sys.which(mode)[[mode]])
-  if ( ! is.null(conf) ) return(conf)
+  if ( ( mode == "java" ) && ( osType() == "mac" ) ) {  # Avoid "No Java runtime present, requesting install." pop-up.
+    label <- "/usr/libexec/java_home helper"
+    home <- try(suppressWarnings(system2("/usr/libexec/java_home",stdout=TRUE,stderr=TRUE)),silent=TRUE)
+    conf <- tryCandidate(if ( ! inherits(home,"try-error") && ( home != "" ) ) file.path(home,"bin",mode) else "")
+    if ( ! is.null(conf) ) return(conf)
+  } else {
+    label <- "PATH environment variable"
+    conf <- tryCandidate(Sys.which(mode)[[mode]])
+    if ( ! is.null(conf) ) return(conf)
+  }
   ###
   if ( mode == "java" ) {
     label <- paste0("R CMD config JAVA")
@@ -216,7 +220,7 @@ extractArchive <- function(archivePath, parentDirectory, directoryName) {
 
 #' @importFrom utils download.file
 #' 
-installSoftware <- function(installPath, software, version, os, bit, verbose=FALSE, downloadFailureCount=0, extractFailureCount=0) {
+installSoftware <- function(installPath, software, version, os, arch, verbose=FALSE, downloadFailureCount=0, extractFailureCount=0) {
   sel <- urls$software == software
   if ( missing(version) ) {
     version <- if ( software == "java" ) {
@@ -232,12 +236,14 @@ installSoftware <- function(installPath, software, version, os, bit, verbose=FAL
     os <-  if ( software == "java" ) osType() else "any"
   }
   sel <- sel & (urls$os == os)
-  if ( missing(bit) ) {
-    bit <- if ( software == "java" ) {
-      if ( Sys.getenv("RSCALA_VERIFY_JAVA_BIT","") != "" ) Sys.getenv("RSCALA_VERIFY_JAVA_BIT","") else osBit()
+  if ( missing(arch) ) {
+    arch <- if ( software == "java" ) {
+      if ( Sys.getenv("RSCALA_VERIFY_JAVA_ARCH","") != "" ) Sys.getenv("RSCALA_VERIFY_JAVA_ARCH","") else R.Version()$arch
     } else "any"
   }
-  sel <- sel & (urls$bit == bit)
+  sel <- if ( arch == "any" ) sel else {
+    sel & sapply(urls$arch, function(re) grepl(re,arch))
+  }
   urls2 <- urls[sel, ]
   candidates <- urls2[order(as.numeric(urls2$priority),decreasing=TRUE),"url"]
   if ( verbose ) {
@@ -267,7 +273,7 @@ installSoftware <- function(installPath, software, version, os, bit, verbose=FAL
 
 javaSpecifics <- function(javaCmd,verbose) {
   if ( verbose ) cat("  ... querying Java specifics.\n")
-  response <- system2(javaCmd,"-version",stdout=TRUE,stderr=TRUE)
+  response <- suppressWarnings(system2(javaCmd,"-version",stdout=TRUE,stderr=TRUE))
   # Get version information
   versionRegexp <- '(java|openjdk) version "([^"]*)".*'
   line <- response[grepl(versionRegexp,response)]
@@ -329,8 +335,8 @@ verifyDownloads <- function() {
     for ( efc in 0:0 ) {
       Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
       Sys.setenv(RSCALA_VERIFY_JAVA_VERSION=version)
-      Sys.setenv(RSCALA_VERIFY_JAVA_BIT=32)
-      cat(paste0("----------\nefc=",efc,", software='java', version=",version,", bit=32\n"))
+      Sys.setenv(RSCALA_VERIFY_JAVA_ARCH="i386")
+      cat(paste0("----------\nefc=",efc,", software='java', version=",version,", arch=i386\n"))
       scalaConfig(download="java")
       s <- scala()
       cat(s * '"OKAY"',"\n\n")
@@ -341,7 +347,7 @@ verifyDownloads <- function() {
     for ( efc in 0:2 ) {
       Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
       Sys.setenv(RSCALA_VERIFY_JAVA_VERSION=version)
-      Sys.setenv(RSCALA_VERIFY_JAVA_BIT=64)
+      Sys.setenv(RSCALA_VERIFY_JAVA_ARCH="x86_64")
       cat(paste0("----------\nefc=",efc,", software='java', version=",version,"\n"))
       scalaConfig(download="java")
       s <- scala()
@@ -349,7 +355,7 @@ verifyDownloads <- function() {
       close(s)
     }
   }
-  for ( version in c("2.11","2.13.0-RC1","2.12") ) {
+  for ( version in c("2.11","2.13.0-RC2","2.12") ) {
     for ( efc in 0:1 ) {
       Sys.setenv(RSCALA_VERIFY_EXTRACT_FAILURE_COUNT=efc)
       Sys.setenv(RSCALA_VERIFY_SCALA_VERSION=version)
@@ -370,5 +376,5 @@ verifyDownloads <- function() {
       cat("\n")
     }
   }
-  Sys.unsetenv(c("RSCALA_VERIFY_EXTRACT_FAILURE_COUNT","RSCALA_VERIFY_JAVA_VERSION","RSCALA_VERIFY_SCALA_VERSION","RSCALA_VERIFY_SBT_VERSION","RSCALA_VERIFY_JAVA_BIT"))
+  Sys.unsetenv(c("RSCALA_VERIFY_EXTRACT_FAILURE_COUNT","RSCALA_VERIFY_JAVA_VERSION","RSCALA_VERIFY_SCALA_VERSION","RSCALA_VERIFY_SBT_VERSION","RSCALA_VERIFY_JAVA_ARCH"))
 }
